import sys
import os
sys.path.extend(['..'])


from dbaccess import *

testdb = Database('sqlite','test.db3')

cursor = testdb.db.connection.cursor()


cursor.execute('drop table if exists lov')
cursor.execute("""
CREATE TABLE [lov] (
  [lovID] integer PRIMARY KEY AUTOINCREMENT, 
  [lovClass] varchar(32), 
  [lovKey] varchar(64), 
  [lovValue] VARCHAR(255), 
  [lovFlag1] varchar(255), 
  [lovFlag2] varchar(255), 
  [lovFlag3] varchar(255), 
  [lovFlag4] varchar(255), 
  [lovRemark] varchar(255),
  [lovNumber] Double,
  [lovDate] DATETIME
  );
  """)
cursor.execute("insert into lov (lovClass,lovKey,lovValue) values ('C1','1','eins')")
cursor.execute("insert into lov (lovClass,lovKey,lovValue) values ('C1','2','zwei')")
cursor.execute("insert into lov (lovClass,lovKey,lovValue) values ('C1','3','drei')")
cursor.close()

class LovDomain(Domain) :
   lovID     = None
   lovClass  = None
   lovKey    = None
   lovValue  = None
   lovFlag1  = None
   lovFlag2  = None
   lovFlag3  = None
   lovFlag4  = None
   lovRemark = None
   lovDate   = None

   meta = {
      'tablename':'lov',
      'primarykey':'lovID',
      
      'fields':{
         'lovID'       : {'dbfield':'lovID',          'type':'Integer'},
         'lovClass'    : {'dbfield':'lovClass',       'type':'String'},
         'lovKey'      : {'dbfield':'lovKey',         'type':'String'},
         'lovValue'    : {'dbfield':'lovValue',       'type':'String'},
         'lovFlag1'    : {'dbfield':'lovFlag1',       'type':'String'},
         'lovFlag2'    : {'dbfield':'lovFlag2',       'type':'String'},
         'lovFlag3'    : {'dbfield':'lovFlag3',       'type':'String'},
         'lovFlag4'    : {'dbfield':'lovFlag4'}, # Absichtlich kein Typ
         'lovRemark'   : {'dbfield':'lovRemark',      'type':'Email'},
         'lovNumber'   : {'dbfiled':'lovNumber',      'type':'Float'},
         'lovDate'     : {'dbfiled':'lovDate',        'type':'Date'}
         }     
      }

   def getLovValueUpper(self):
      return self.lovValue.upper()

def hasException (code):
   """Prueft ob bei uebergebenen Code eine Exception auftritt"""

   retval = False
   try:
      eval(code)
   except:
      return True
   return False

lov = LovDomain(testdb)

# Pruefen ob None zugelassen wird (typecheckNoneAllowed = True
lov.lovID = None

# ### Test Typensichere Zuweisung
# 

# None nicht erlaubt...
lov.typecheckNoneAllowed = False
assert(hasException('lov.lovID = None'))

# Integer
lov.lovID = 23

# Prufen ob ein Fehler auftritt
assert(hasException("lov.lovID = 'abc'"))
assert(hasException("lov.lovID = 23.42"))

assert(hasException("lov.lovRemark = 'test@nirgendwo.at'"))

assert(hasException("lov.lovRemark = 23.42"))
lov.lovClass = 'xxx'
lov.lovNumber = 23.42

assert(hasException("lov.lovNumber = 23"))

# Ohne Typepruefung
# Es darf kein Fehler auftreten
lov.lovFlag4 = 23

# Typenpruefung abschalten
lov.typecheckStrict = False
lov.lovID = 'abc'

# wieder einschalten
lov.typecheckStrict = True

# ### DB Checks...
# 

# Test count
assert(lov.count()==3)
assert(lov.count(where='lovClass = "C1" ') == 3)
assert(lov.count(where='lovValue = "eins" ') == 1)

# Test Summenfunktion
assert(lov.sum('lovID') == 6)
assert(lov.sum('lovID',where="lovValue='drei'") == 3)

#test Min
assert(lov.min('lovID') == 1)
assert(lov.min('lovID',where="lovValue='drei'") == 3)

#test Max
assert(lov.max('lovID') == 3)
assert(lov.max('lovID',where="lovValue='drei'") == 3)

#test Avg
assert(lov.avg('lovID') == 2)
assert(lov.avg('lovID',where="lovValue='drei'") == 3)


cnt=0
for l in lov.eachDomain():
   cnt+=1
assert(cnt==3)


# Test eachDomain mit where
cnt = 0
for l in lov.eachDomain(where='lovKey=1'):
   cnt+=1
assert(cnt==1)

# Test eachDomain mit where
cnt = 0
for l in lov.eachDomain(where='lovClass = "C1"'):
   cnt+=1
assert(cnt==3)

# Test eachDomain mit orderby
cnt = 0
oldID = ''
for l in lov.eachDomain(orderby='lovValue'):
   if l.lovValue > oldID:
      cnt+=1
      oldID = l.lovValue
assert(cnt==3)


# Test: get mit PK
lov.get(1)
assert(lov.lovID == 1)


# Test: get mit where
lov.get(where="lovValue = 'zwei'")
assert(lov.lovValue=='zwei')

# Test notOK
lov.get(99)
assert (not lov.isOk)

# Test Domain Methode
lov.get(2)
assert(lov.getLovValueUpper()=='zwei'.upper())

# Test Loeschen aktueller Datensatz
lov.get(2)
lov.delete()
lov.get(2)
assert(not lov.isOk)

# Test delete in unbekannten Datensatz
lov.clear()
cnt = 0

assert(hasException('lov.delete()'))

# Test deleteAll
lov.deleteAll(where='lovID=3')
lov.get(3)
assert(not lov.isOk)


# Test clear
lov.get(1)
lov.clear()
assert(lov.lovID==None)

# Test Insert
lov.clear()
lov.lovClass = 'C2'
lov.lovKey   = 'k2'
lov.lovValue = 'v2'
lov.insert()
lov.clear()
lov.get(4)
assert(lov.lovKey=='k2')

# Test Insert
lov.clear()
lov.lovClass = 'C2'
lov.lovKey   = 'k3'
lov.lovValue = 'v3'
lov.insert()
lov.clear()
lov.get(5)
assert(lov.lovKey=='k3')

# Test Update
lov.clear()
lov.get(5)
lov.lovFlag1='update'
lov.update()
lov.get(5)
assert(lov.lovFlag1=='update')

# Test: loesche aller Datensaetze
lov.deleteAll(where='(1=1)')
assert(lov.count() == 0)

# Test Datum
lov.clear()
lov.lovDate = '1.1.2011'
lov.insert()
lov.get(6)
assert(lov.lovDate == '2011-01-01T00:00:00')

assert(hasException("lov.lovDate = '31.2.2011'"))
assert(hasException("lov.lovDate = 'keindatum'"))

lov.clear()
lov.lovDate = '1.1.2011 10:22'
lov.insert()
lov.get(7)
assert(lov.lovDate == '2011-01-01T10:22:00')


print 'done'
