# -*- coding: iso-8859-15 -*-
import sys

# ### CONFIG
# Importiert die userseitige Konfigurationsklasse
#
from conf.config import Config

# #####
# 
# Abstrakte Klasse fuer die Menudarstellung
# Sie wird normalerweise von der Klasse Menu im WEB-INF/conf/menu
# importiert.
# 
# 
# 
class MenuBase:

   # ### GLOBALS
   #
   
   authen = None

   # Container fuer Menuedarstellung
   tplMenuContainer='<ul class="menu">\n%(entry)s</ul>\n'
   tplMenuEntry=' <li class="menu">&rarr;&nbsp;<a class="menu" href="%(homeurl)s?path=%(entry)s">%(text)s</a></li>\n'

   # Container fuer Backlink Darstellung
   tplBacklinkContainer = '<span style="font-size:smaller">Du bist hier: %(entry)s</span>'
   tplBacklinkEntry = '<a href="%(root)s?path=%(path)s"  class="backlink-menu">%(text)s</a>'
   tplBacklinkSep = ' &rarr; '

   # Text des Zurueck Buttons
   returnText = 'zur&uuml;ck!'
   
   returnPath = ''
   
   # URL auf HOME, wird zum
   # erzeugen des URL mit Path verwendet
   # meist 'start.py'
   #
   homeurl=None
   
   # aktueller Inhalt des CGI Parameters path
   # wird von main.py gesetzt
   path=None

   # Menueintraege
   # HINT:
   #   Wird von der Klasse menu
   #   im Verzeichnis conf bei der Initialisierung gesetzt
   #
   entries = []

   # 
   # Konstante fuer URL Kennung in Controller
   #
   controllerIsUrl = '@url:'

   error = ''

   def getEntry(self,path,dieifnotfound=True):
      '''Gibt ein Entry referenziert auf path zurueck
         Wenn "dieifnotfound" gesetzt ist
            wirft Exception wenn nicht auffindbar
         sonst
            wird None zurueckgegeben
         '''
      # durchlaufen der Menueeintraege
      #
      for entry in self.entries:
         # Wenn Eintrag gefunden, wird 
         # das Dictionary zurueckgegeben
         if entry['path'] == path:
            return entry
      # Wurde der Parameter dieifnotfound
      # gesetzt wird, wenn der Eintrag nicht 
      # gefunden wurde eine Exception geworfen
      if dieifnotfound:
         return None
         raise Exception("Kann keinen Einrag basierend auf '%(path)s' finden" % {path:path})
      
      # Wenn nicht gefunden, None zurueckgeben
      return None

    
   def truncatepath(self,path):
      'Abschneiden des letzen Teils des Pfades'
      
      # Wenn kein Pfad definiert
      # abbruch
      if path == None:
         return

      # Finde den letzten Pfadtrenner
      pos = path.rfind('/')
      
      # wurde dieser nicht gefunden,
      # wird ein Leerstring zurueckgeliefert
      if pos == -1:
         return ''
      
      # ist nur mehr ein Eintrag vorhanden
      # diesen Zurueckgeben
      if pos == 0:
         return '/'
      
      # Wenn wir bis hierher gekommen sind
      # letzen Eintrag abschneiden und das Ergebnis
      # zurueckliefern
      return path[0:pos]

   def getHtml(self,entry):
      'Liefert das HTML Snippet fuer die Anzeige'

      controller = entry.get('controller')
      if controller == None: controller = ''
      if controller.startswith(self.controllerIsUrl):
         formatstring = self.tplMenuEntry
         controller = controller.replace(self.controllerIsUrl,'')
         formatstring = formatstring.replace('%(homeurl)s?path=%(entry)s',controller)
         retval = formatstring % {'text':entry.get('text')}
         return retval

      retval = self.tplMenuEntry % {'homeurl':self.homeurl,'entry':entry['path'],'text':entry['text']}
      return retval

   def hasEntryRight(self,entry):
      'Liefert True wenn der User das Recht hat den Menueintrag aufzurufen'
      
      # Rechte des Menueeintrags
      rights = entry.get('rights')

      # Flag 
      showIt = False
      # wurden Rechte gesetzt
      if rights != None or rights==[]:   
         for menRight in rights:

            if self.authen.hasRight(right=menRight):
               showIt = True
      else:
         # Wenn keine Rechte im Menueintrag
         # auf jeden Fall anzeigen
         showIt = True

      return showIt
      

   def navigation(self):
      'Gibt das gesamte Menue zurueck'            
      retval = ''
      
      # zusammenstellen Menue
      for entry in self.entries:
         # Wenn kein Texteintrag nicht anzeigen
         if entry.get('text') != None:
            # Letzten Pfadeintrag abschneiden
            truncpath = self.truncatepath(entry['path'])
            # Im aktuellen Pfad
            if truncpath == self.path:
               # Ueberpruefen ob User berechtigt ist
               showIt = self.hasEntryRight(entry)
               if showIt:
                  retval = retval + self.getHtml(entry) 
	            
	                    
      # Wenn nicht oberste Ebene
      # wird Return Schaltflaeche ausgegeben
      if self.path != '/root':
         if self.returnPath == '':
            self.returnPath = self.path
         
         # Pruefen ob berechtigt den Menupunkt anzuzeigen
         #
         retval = retval + self.getHtml({'path':self.truncatepath(self.returnPath),'text':self.returnText})
      return self.tplMenuContainer % {'entry':retval}

   def backlink(self):
      retval = ''
      
      path = self.path  #self.truncatepath(path=self.path)
      while (path != '/'):
         entry = self.getEntry(path)
         if entry==None:
            text = ''
         else:
            text = entry.get('text')

         if text != None:
            url = self.tplBacklinkEntry % {'root':'start.py',
                                           'path':path,
                                           'text':text
                                           }

            # Bei aktuellem Pfad keinen Trenner einfuegen
            if path != self.path:
               url += self.tplBacklinkSep
            # Resultat zusammenbauen         
            retval = url + retval

         path = self.truncatepath(path)

      return self.tplBacklinkContainer % {'entry':retval}

   def getControllerName(self):
      'liefert den Namen des Controller basierend auf path'
      entry = self.getEntry(self.path,dieifnotfound=False)
      if entry == None:
         entry = {}

      if 'controller' in entry:
         auxPath = self.path
         auxPath = auxPath.replace('/','.')
         
         return ("mvc%(path)s.%(controllername)s" % {'path':auxPath,'controllername':entry.get('controller')})
      else:
         return None

   def getController(self):
      '''Liefert eine Controller Objekt
         Es wird angenommen, dass der Controller im Dateisystem
         mit dem Pfad der CGI variable path liegt.

         Desweiteren wird ueberprueft, ob der Benutzer das Recht hat
         diesen Controller aufzurufen.
         '''
      
      self.error = ''

      entry = self.getEntry(self.path)

      if entry == None:
         entry = {}

      if 'controller' in entry:
         if entry.get('controller') == None:
            return None

         if entry.get('controller').startswith('@url:'):
            raise(Exception('***'))
            return None

         if not self.hasEntryRight(entry):
            return None

         cpath = self.path.replace('/','.')
         # vorlaufender Punkt entfernen
         cpath = cpath.lstrip('.')
         sImport = "from %(from)s import %(import)s as Controller" % {
            'from':self.getControllerName(),
            'import':entry.get('controller')
            }

         try:
            exec (sImport)
         except Exception, e:
            raise (Exception("%(import)s -- Kann Controller '%(controller)s.py' in path '%(path)s' nicht initalisieren!\nErrormsg:%(errormsg)s" % {
                 'controller':entry.get('controller'),
                 'path':self.path,
                 'import':sImport,
                 'errormsg':e}))
            return None

         controller = Controller()
         controller.path=self.path
         return controller
      else :
         return None
   